/*
 *  Copyright (C) 2007 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_ttf.h>

#include <pspctrl.h>
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspiofilemgr.h>
#include <psppower.h>

#include "psp_global.h"
#include "psp_danzeff.h"
#include "psp_sdl.h"
#include "psp_menu.h"
#include "psp_battery.h"
#include "psp_help.h"
#include "psp_irkeyb.h"
#include "psp_fmgr.h"
#include "psp_editor.h"

# define MENU_FILENAME     0
# define MENU_WIDTH        1
# define MENU_HEIGHT       2
# define MENU_FG_COLOR     3
# define MENU_BG_COLOR     4
# define MENU_FONT_SIZE    5
# define MENU_DOS_MODE     6
# define MENU_EXPAND_TAB   7
# define MENU_TAB_SIZE     8
# define MENU_MOVE_CURSOR  9
# define MENU_CLOCK       10

# define MENU_HELP        11
# define MENU_NEW         12
# define MENU_LOAD        13
# define MENU_SAVE        14

# define MENU_BACK        15
# define MENU_EXIT        16

# define MAX_MENU_ITEM (MENU_EXIT + 1)

  static menu_item_t menu_list[] =
  {
    { "Filename   : " },
    { "Width      : " },
    { "Height     : " },
    { "Text color : " },
    { "Back color : " },
    { "Font size  : " },
    { "Dos mode   : " },
    { "Expand tab : " },
    { "Tab size   : " },
    { "Cursor     : " },
    { "CPU Clock  : " },

    { "Help" },

    { "New" },
    { "Load" },
    { "Save" },
    { "Back" },

    { "Exit" }
  };

  static int cur_menu_id = MENU_NEW;
  static int psp_cpu_clock = 222;
  static int psp_font_size = PSP_FONT_6X10;
  static int dos_mode = 0;
  static int expand_tab = 0;
  static int tab_stop = 0;
  static int move_on_text = 0;
  static int fg_color = 0;
  static int bg_color = 0;
  static int screen_w = 0;
  static int screen_h = 0;

  static int EDITOR_SCREEN_HEIGHT = 0;
  static int EDITOR_SCREEN_WIDTH  = 0;

void
string_fill_with_space(char *buffer, int size)
{
  int length = strlen(buffer);
  int index;

  for (index = length; index < size; index++) {
    buffer[index] = ' ';
  }
  buffer[size] = 0;
}

void
psp_display_screen_menu_battery(void)
{
  char buffer[64];

  int Length;
  int color;

  snprintf(buffer, 50, " [%s] ", psp_get_battery_string());
  Length = strlen(buffer);

  if (psp_is_low_battery()) color = PSP_MENU_RED_COLOR;
  else                      color = PSP_MENU_BORDER_COLOR;

  psp_sdl_back_print(240 - ((8*Length) / 2), 6, buffer, color);
}

# define FILENAME_FIELD_WIDTH      40

static void 
psp_display_screen_menu(void)
{
  char buffer[512];

  int menu_id = 0;
  int color   = 0;
  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  {
    psp_sdl_blit_menu();

    psp_sdl_draw_rectangle(10,10,459,249,PSP_MENU_BORDER_COLOR,0);
    psp_sdl_draw_rectangle(11,11,457,247,PSP_MENU_BORDER_COLOR,0);

    psp_sdl_back_print(  30, 6, " Menu ", PSP_MENU_NOTE_COLOR);

    psp_display_screen_menu_battery();

    psp_sdl_back_print(30, 254, " O/X: Valid  Sel: Back  Start: Keyboard ", 
                       PSP_MENU_BORDER_COLOR);

    psp_sdl_back_print(370, 254, " By Zx-81 ",
                       PSP_MENU_AUTHOR_COLOR);
  }
  
  x      = 20;
  y      = 25;
  y_step = 10;
  
  for (menu_id = 0; menu_id < MAX_MENU_ITEM; menu_id++) {
    color = PSP_MENU_TEXT_COLOR;
    if (cur_menu_id == menu_id) color = PSP_MENU_SEL_COLOR;
    else 
    if (menu_id == MENU_EXIT) color = PSP_MENU_WARNING_COLOR;
    else
    if (menu_id == MENU_HELP) color = PSP_MENU_GREEN_COLOR;

    psp_sdl_back_print(x, y, menu_list[menu_id].title, color);

    if (menu_id == MENU_FILENAME) {
      int length = strlen(PSPWRITE.edit_filename);
      int first = 0;
      if (length > (FILENAME_FIELD_WIDTH-1)) {
        first = length - (FILENAME_FIELD_WIDTH-1);
      }
      sprintf(buffer, "%s", PSPWRITE.edit_filename + first);
      if (menu_id == cur_menu_id) strcat(buffer, "_");
      string_fill_with_space(buffer, FILENAME_FIELD_WIDTH+1);
      psp_sdl_back_print(120, y, buffer, color);

    } else
    if (menu_id == MENU_MOVE_CURSOR) {
      if (move_on_text) strcpy(buffer, "only on text");
      else              strcpy(buffer, "everywhere");
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);

    } else
    if (menu_id == MENU_WIDTH) {
      sprintf(buffer, "%d", screen_w);
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);

    } else
    if (menu_id == MENU_HEIGHT) {
      sprintf(buffer, "%d", screen_h);
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);

    } else
    if (menu_id == MENU_FG_COLOR) {
      sprintf(buffer,"%s", editor_colors_name[fg_color]);
      color = editor_colors[fg_color];
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);

    } else
    if (menu_id == MENU_BG_COLOR) {
      sprintf(buffer,"%s", editor_colors_name[bg_color]);
      color = editor_colors[bg_color];
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);

    } else
    if (menu_id == MENU_FONT_SIZE) {
      if (psp_font_size == PSP_FONT_6X10) strcpy(buffer, "6x10");
      else                                strcpy(buffer, "8x8");
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);
    } else
    if (menu_id == MENU_DOS_MODE) {
      if (dos_mode) strcpy(buffer, "yes");
      else                  strcpy(buffer, "no");
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);
    } else
    if (menu_id == MENU_EXPAND_TAB) {
      if (expand_tab) strcpy(buffer, "yes");
      else                     strcpy(buffer, "no");
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);
    } else
    if (menu_id == MENU_TAB_SIZE) {
      sprintf(buffer, "%d", tab_stop);
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);
    } else
    if (menu_id == MENU_CLOCK) {
      sprintf(buffer, "%d", psp_cpu_clock);
      string_fill_with_space(buffer, 32);
      psp_sdl_back_print(120, y, buffer, color);
      y += y_step;

    } else
    if (menu_id == MENU_HELP) {
      y += y_step;
    } else
    if (menu_id == MENU_BACK) {
      y += y_step;
    }

    y += y_step;
  }
}

static void
psp_edit_menu_validate(void)
{
  PSPWRITE.psp_cpu_clock = psp_cpu_clock;
  PSPWRITE.dos_mode      = dos_mode;
  PSPWRITE.expand_tab    = expand_tab;
  PSPWRITE.tab_stop      = tab_stop;
  PSPWRITE.move_on_text  = move_on_text;
  PSPWRITE.fg_color      = fg_color;
  PSPWRITE.bg_color      = bg_color;
  PSPWRITE.screen_h      = screen_h;
  PSPWRITE.screen_w      = screen_w;
  PSPWRITE.psp_font_size = psp_font_size;

  scePowerSetClockFrequency(PSPWRITE.psp_cpu_clock, PSPWRITE.psp_cpu_clock, PSPWRITE.psp_cpu_clock/2);
}

static void
psp_edit_menu_init()
{
  psp_cpu_clock = PSPWRITE.psp_cpu_clock;
  psp_font_size = PSPWRITE.psp_font_size;
  dos_mode      = PSPWRITE.dos_mode;
  expand_tab    = PSPWRITE.expand_tab;
  tab_stop      = PSPWRITE.tab_stop;
  move_on_text  = PSPWRITE.move_on_text;
  fg_color      = PSPWRITE.fg_color;
  bg_color      = PSPWRITE.bg_color;
  screen_h      = PSPWRITE.screen_h;
  screen_w      = PSPWRITE.screen_w;

  if (psp_font_size == PSP_FONT_6X10) {
    EDITOR_SCREEN_WIDTH = EDITOR_SCREEN_WIDTH_6X10;
    EDITOR_SCREEN_HEIGHT = EDITOR_SCREEN_HEIGHT_6X10;
  } else {
    EDITOR_SCREEN_WIDTH = EDITOR_SCREEN_WIDTH_8X8;
    EDITOR_SCREEN_HEIGHT = EDITOR_SCREEN_HEIGHT_8X8;
  }
}

static void
psp_edit_menu_save_config()
{
  int error;

  psp_edit_menu_validate();

  error = psp_global_save_config();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(270, 80, "Config file saved !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back_print(270, 80, "Can't save config file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
}

int
psp_edit_menu_confirm_overwrite()
{
  SceCtrlData c;

  if (PSPWRITE.ask_overwrite) {

    struct stat aStat;
    if (stat(PSPWRITE.edit_filename, &aStat)) {
      PSPWRITE.ask_overwrite = 0;
      return 1;
    }

    psp_display_screen_menu();
    psp_sdl_back_print(200, 70, "X to overwrite ? [] to cancel", PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();

    psp_kbd_wait_no_button();
    do
    {
      sceCtrlReadBufferPositive(&c, 1);
      c.Buttons &= PSP_ALL_BUTTON_MASK;

# ifdef USE_PSP_IRKEYB
      psp_irkeyb_set_psp_key(&c);
# endif

      if (c.Buttons & PSP_CTRL_CROSS) {
        return 1;
      } else 
      if (c.Buttons & PSP_CTRL_SQUARE) {
        return 0;
      }

    } while (c.Buttons == 0);
    psp_kbd_wait_no_button();
  }
  return 0;
}

static int
psp_edit_menu_save()
{
  int error;

  if (PSPWRITE.ask_overwrite) {
    if (! psp_edit_menu_confirm_overwrite()) {
      return 0;
    }
  }
  PSPWRITE.ask_overwrite = 0;
  error = psp_editor_save();

  if (! error) /* save OK */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(140, 160, "File saved !", 
                       PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  else 
  {
    psp_display_screen_menu();
    psp_sdl_back_print(140, 160, "Can't save file !", 
                       PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  return error;
}

int
psp_edit_menu_confirm_save()
{
  SceCtrlData c;

  if (PSPWRITE.is_modified) {

    psp_display_screen_menu();
    psp_sdl_back_print(220, 70, "Save last change ? X to save", PSP_MENU_WARNING_COLOR);
    psp_sdl_back_print(220, 80, "O to ignore, [] to cancel", PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();

    psp_kbd_wait_no_button();
    do
    {
      sceCtrlReadBufferPositive(&c, 1);
      c.Buttons &= PSP_ALL_BUTTON_MASK;

# ifdef USE_PSP_IRKEYB
      psp_irkeyb_set_psp_key(&c);
# endif

      if (c.Buttons & PSP_CTRL_CROSS) {
        if (psp_edit_menu_save()) {
          /* failed -> cancel */
          return 0;
        }
      } else 
      if (c.Buttons & PSP_CTRL_SQUARE) {
        return 0;
      } else 
      if (c.Buttons & PSP_CTRL_CIRCLE) {
        break;
      }

    } while (c.Buttons == 0);
    psp_kbd_wait_no_button();
  }
  return 1;
}

int
psp_edit_menu_exit(void)
{
  if (! psp_edit_menu_confirm_save()) {
    return 0;
  }

  SceCtrlData c;

  psp_display_screen_menu();
  psp_sdl_back_print(270, 70, "Press X to exit !", PSP_MENU_WARNING_COLOR);
  psp_sdl_back_print(220, 80, "Press O to save config & exit", PSP_MENU_WARNING_COLOR);
  psp_sdl_flip();

  psp_kbd_wait_no_button();

  do
  {
    sceCtrlReadBufferPositive(&c, 1);
    c.Buttons &= PSP_ALL_BUTTON_MASK;

    if (c.Buttons & PSP_CTRL_CROSS) {
      psp_sdl_exit(0);
    } else
    if (c.Buttons & PSP_CTRL_CIRCLE) {
      psp_edit_menu_save_config();
      psp_sdl_exit(0);
    }

  } while (c.Buttons == 0);

  psp_kbd_wait_no_button();

  return 0;
}

int
psp_edit_menu_new(void)
{
  if (psp_edit_menu_confirm_save()) {
    psp_editor_new();
  }
  return 0;
}

int
psp_edit_menu_load(void)
{
  int ret;

  if (! psp_edit_menu_confirm_save()) {
    return 0;
  }
  ret = psp_fmgr_menu(PSP_FMGR_FORMAT_TXT);
  if (ret ==  1) /* load OK */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(270,  60, "File loaded !", PSP_MENU_NOTE_COLOR);
    psp_sdl_flip();
    sleep(1);
    return 1;
  }
  else 
  if (ret == -1) /* Load Error */
  {
    psp_display_screen_menu();
    psp_sdl_back_print(270,  60, "Can't load file !", PSP_MENU_WARNING_COLOR);
    psp_sdl_flip();
    sleep(1);
  }
  return 0;
}

void
psp_edit_menu_color( int step )
{
  if (step < 0) {
    fg_color--;
    if (fg_color < 0) fg_color = COLOR_IMAGE-1;
  } else {
    fg_color++;
    if (fg_color >= COLOR_IMAGE) fg_color = 0;
  }
}

void
psp_edit_menu_font_size( int step ) 
{
  psp_font_size = ! psp_font_size;

  if (psp_font_size == PSP_FONT_6X10) {
    EDITOR_SCREEN_WIDTH = EDITOR_SCREEN_WIDTH_6X10;
    EDITOR_SCREEN_HEIGHT = EDITOR_SCREEN_HEIGHT_6X10;
  } else {
    EDITOR_SCREEN_WIDTH = EDITOR_SCREEN_WIDTH_8X8;
    EDITOR_SCREEN_HEIGHT = EDITOR_SCREEN_HEIGHT_8X8;
  }

  if (screen_w > EDITOR_SCREEN_WIDTH) screen_w = EDITOR_SCREEN_WIDTH;
  if (screen_h > EDITOR_SCREEN_HEIGHT) screen_h = EDITOR_SCREEN_HEIGHT;
}

void
psp_edit_menu_width( int step )
{
  if (step < 0) {
    screen_w--;
    if (screen_w < EDITOR_SCREEN_MIN_WIDTH) {
      screen_w = EDITOR_SCREEN_WIDTH;
    }
  } else {
    screen_w++;
    if (screen_w > EDITOR_SCREEN_WIDTH) {
      screen_w = EDITOR_SCREEN_MIN_WIDTH;
    }
  }
}

#define MAX_CLOCK_VALUES 5
static int clock_values[MAX_CLOCK_VALUES] = { 133, 222, 266, 300, 333 };

static void
psp_edit_menu_clock(int step)
{
  int index;
  for (index = 0; index < MAX_CLOCK_VALUES; index++) {
    if (psp_cpu_clock == clock_values[index]) break;
  }
  if (step > 0) {
    index++;
    if (index >= MAX_CLOCK_VALUES) index = 0;
    psp_cpu_clock = clock_values[index];

  } else {
    index--;

    if (index < 0) index = MAX_CLOCK_VALUES - 1;
    psp_cpu_clock = clock_values[index];
  }
}


void
psp_edit_menu_height( int step )
{
  if (step > 0) {
    screen_h++;
    if (screen_h > EDITOR_SCREEN_HEIGHT) {
      screen_h = EDITOR_SCREEN_MIN_HEIGHT;
    }
  } else {
    screen_h--;
    if (screen_h < EDITOR_SCREEN_MIN_HEIGHT) {
      screen_h = EDITOR_SCREEN_HEIGHT;
    }
  }
}

void
psp_edit_menu_bg_color( int step )
{
  if (step > 0) {
    bg_color++;
    if (bg_color >= EDITOR_MAX_COLOR) bg_color = 0;
  } else {
    bg_color--;
    if (bg_color < 0) bg_color = EDITOR_MAX_COLOR-1;
  }
}


void
psp_edit_menu_tabsize( int step )
{
  if (step < 0) {
    tab_stop--;
    if (tab_stop < 1) tab_stop = 10;
  } else {
    tab_stop++;
    if (tab_stop > 10) tab_stop = 1;
  }
}

void
psp_edit_menu_filename_add_char(int a_char)
{
  int length = strlen(PSPWRITE.edit_filename);
  if (length < (MAX_PATH-2)) {
    PSPWRITE.edit_filename[length] = a_char;
    PSPWRITE.edit_filename[length+1] = 0;
  }
}

void
psp_edit_menu_filename_clear()
{
  PSPWRITE.edit_filename[0] = 0;
}

void
psp_edit_menu_filename_del()
{
  int length = strlen(PSPWRITE.edit_filename);
  if (length >= 1) {
    PSPWRITE.edit_filename[length - 1] = 0;
  }
}

int 
psp_edit_main_menu(void)
{
  SceCtrlData c;
  long        new_pad;
  long        old_pad;

  int         last_time;
  int         end_menu;

  int         danzeff_mode = 0;
  int         danzeff_key;

  int         step = 0;

  psp_kbd_wait_no_button();
  psp_sdl_select_font_8x8();

  psp_edit_menu_init();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;

# ifdef USE_PSP_IRKEYB
  int irkeyb_key = PSP_IRKEYB_EMPTY;
# endif

  while (! end_menu)
  {
    psp_display_screen_menu();

    if (danzeff_mode) {
      danzeff_moveTo(-50, -50);
      danzeff_render();
    }
    psp_sdl_flip();

    while (1) {

      sceCtrlPeekBufferPositive(&c, 1);
      c.Buttons &= PSP_ALL_BUTTON_MASK;

# ifdef USE_PSP_IRKEYB
      irkeyb_key = psp_irkeyb_set_psp_key(&c);
# endif
      new_pad = c.Buttons;

      if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > PSP_MENU_MIN_TIME)) {
        last_time = c.TimeStamp;
        old_pad = new_pad;
        break;
      }
# ifdef USE_PSP_IRKEYB
      if (irkeyb_key != PSP_IRKEYB_EMPTY) break;
# endif
    }

# ifdef USE_PSP_IRKEYB
    irkeyb_key = psp_irkeyb_read_key();

    if (irkeyb_key > 0) {
      if (irkeyb_key > ' ') {
        psp_edit_menu_filename_add_char(irkeyb_key);
      } else
      if (irkeyb_key == 0xc) {
        psp_edit_menu_filename_clear(irkeyb_key);
      } else
      if (irkeyb_key == 0x8) {
        psp_edit_menu_filename_del(irkeyb_key);
      } else
      if (irkeyb_key == 0xd) {
        new_pad |= PSP_CTRL_CROSS;
      }
      irkeyb_key = PSP_IRKEYB_EMPTY;
    }
# endif
    if (danzeff_mode) {

      danzeff_key = danzeff_readInput(c);
      if (danzeff_key > DANZEFF_START) {
        /* Disable utf8 */
        danzeff_key &= 0x7f;
        if (danzeff_key > ' ') {
          psp_edit_menu_filename_add_char(danzeff_key);
        } else
        if (danzeff_key == DANZEFF_CLEAR) {
          psp_edit_menu_filename_clear();
        } else
        if (danzeff_key == DANZEFF_DEL) {
          psp_edit_menu_filename_del();
        }

      } else 
      if ((danzeff_key == DANZEFF_START ) || 
          (danzeff_key == DANZEFF_SELECT)) 
      {
        danzeff_mode = 0;
        old_pad = new_pad = 0;

        psp_kbd_wait_no_button();
      }
      continue;
    }

    if ((c.Buttons & (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) ==
        (PSP_CTRL_LTRIGGER|PSP_CTRL_RTRIGGER|PSP_CTRL_START)) {
      /* Exit ! */
      psp_sdl_exit(0);
    } else
    if ((new_pad & (PSP_CTRL_CROSS|PSP_CTRL_RIGHT)) || 
        (new_pad & (PSP_CTRL_CIRCLE|PSP_CTRL_LEFT)))
    {
      if (new_pad & (PSP_CTRL_RIGHT|PSP_CTRL_CROSS)) {
        step = 1;
      } else
      if (new_pad & (PSP_CTRL_LEFT|PSP_CTRL_CIRCLE)) {
        step = -1;
      }

      switch (cur_menu_id ) 
      {
        case MENU_LOAD      : if (psp_edit_menu_load()) {
                                end_menu = 1;
                              }
        break;              

        case MENU_NEW       : psp_edit_menu_new();
                              end_menu = 1;
        break;              

        case MENU_SAVE      : psp_edit_menu_save();
        break;              

        case MENU_BACK      : end_menu = 1;
        break;

        case MENU_EXIT      : psp_edit_menu_exit();
        break;

        case MENU_HELP      : psp_help_menu();
                              old_pad = new_pad = 0;
        break;              

        case MENU_TAB_SIZE : psp_edit_menu_tabsize( step );
        break;              
        case MENU_FG_COLOR : psp_edit_menu_color( step );
        break;              
        case MENU_BG_COLOR : psp_edit_menu_bg_color( step );
        break;              
        case MENU_WIDTH    : psp_edit_menu_width( step );
        break;              
        case MENU_FONT_SIZE: psp_edit_menu_font_size( step );
        break;              
        case MENU_HEIGHT   : psp_edit_menu_height( step );
        break;              
        case MENU_DOS_MODE : dos_mode = ! dos_mode;
        break;              
        case MENU_MOVE_CURSOR : move_on_text = ! move_on_text;
        break;              
        case MENU_EXPAND_TAB : expand_tab = ! expand_tab;
        break;              
        case MENU_CLOCK    : psp_edit_menu_clock( step );
        break;              
        case MENU_FILENAME : if (step < 0) psp_edit_menu_filename_del();
        break;
      }

    } else
    if(new_pad & PSP_CTRL_START) {
      danzeff_mode = 1;
    } else
    if(new_pad & PSP_CTRL_UP) {

      if (cur_menu_id > 0) cur_menu_id--;
      else                 cur_menu_id = MAX_MENU_ITEM-1;

    } else
    if(new_pad & PSP_CTRL_DOWN) {

      if (cur_menu_id < (MAX_MENU_ITEM-1)) cur_menu_id++;
      else                                 cur_menu_id = 0;

    } else  
    if(new_pad & PSP_CTRL_SQUARE) {
      /* Cancel */
      end_menu = -1;
    } else 
    if(new_pad & PSP_CTRL_SELECT) {
      /* Back to Console */
      end_menu = 1;
    } else
    if(new_pad & PSP_CTRL_RTRIGGER) {
      /* TO_BE_DONE ! */
    } else
    if(new_pad & PSP_CTRL_LTRIGGER) {
      /* TO_BE_DONE ! */
    }
  }

  if (end_menu > 0) {
    psp_edit_menu_validate();
  }
 
  psp_editor_update_column();
  psp_editor_update_line();

  psp_kbd_wait_no_button();

  return (end_menu == 2);
}

